%Copyright (C) 2005  Lastenet-Ramel
%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function [imout]=dct2(imin,m,n)
%
% [imout]=dct2(imin,m,n)
%
% Compute two-dimensional discrete cosine transform
% 
% - imout = dct2(imin) returns the two-dimensional discrete cosine transform of imin.
% The matrix imout is the same size as A and contains the discrete cosine transform
% coefficients imout(p,q).
% 
% - imout = dct2(imin,m,n) or imout = dct2(imin,[m n]) pads the matrix imin with 0's
% to size m-by-n before transforming. If m or n is smaller than the corresponding
% dimension of imin, dct2 truncates imin.

[p,q]=size(imin);

if (nargin>3 | nargin<1)
	disp('too many or few arguments');
	break;
end

if (nargin==2)
	if (length(size (m))==2)
		n=m(2);
		m=m(1);
	else 
	disp('too many or few arguments');
	break
	end
end

if(nargin>1)

	if (m<=p & n<=q )
		A=imin(1:m,1:n);
	end

	if (m<=p & n>q )
		A=[imin(1:m,1:n) zeros(m,n-q)];
	end

	if (m>p & n<=q )
		A=[imin(1:m,1:n); zeros(m-q,n)];
	end
	
	if (m>p & n>q )
		A=zeros(m,n);
		A(1:p,1:q)=imin;
	end
end
	
if(nargin==1)
	A=imin;
end

[NbX,NbY]=size(A);

I=[0:NbX-1];
J=[0:NbY-1];

mat_cos1=cos(((pi*I')/(2*NbX))*(2*I+1));
mat_cos2=cos(((pi*J')/(2*NbY))*(2*J+1));

imout=[];

alpha1=sqrt(2/NbX);
alpha2=sqrt(2/NbY);

imout=alpha1*alpha2*mat_cos1*A*mat_cos2';
imout(1,:)=imout(1,:)/sqrt(2);
imout(:,1)=imout(:,1)/sqrt(2);
