%Copyright (C) 2005  Lastenet-Ramel
%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

function [imout]=idct2(imin,m,n)
%
% [imout]=idct2(imin,m,n)
%
% Compute two-dimensional inverse discrete cosine transform
% 
% - imout = idct2(imin) returns the two-dimensional inverse discrete cosine transform (DCT)
% of imin.
% 
% - imout = idct2(imin,m,n) or imout = idct2(imout,[m n]) pads imin with 0's to size
% m-by-n before transforming. If [m n] < size(imin), idct2 crops imin before transforming.
% 
% For any imin, idct2(dct2(imin)) equals imin to within roundoff error.

[p,q]=size(imin);

if (nargin>3 | nargin<1)
	disp('too many or few arguments');
	break;
end

if (nargin==2)
	if (length(size (m))==2)
		n=m(2);
		m=m(1);
	else 
	disp('too many or few arguments');
	break
	end
end

if(nargin>1)

	if (m<=p & n<=q )
		A=imin(1:m,1:n);
	end

	if (m<=p & n>q )
		A=[imin(1:m,1:n) zeros(m,n-q)];
	end

	if (m>p & n<=q )
		A=[imin(1:m,1:n); zeros(m-q,n)];
	end
	
	if (m>p & n>q )
		A=zeros(m,n);
		A(1:p,1:q)=imin;
	end
end
	
if(nargin==1)
	A=imin;
end

[NbX,NbY]=size(A);


M=[0:NbX-1];
N=[0:NbY-1];
P=[0:NbX-1];
Q=[0:NbY-1];

mat_cos1=cos(((pi*P')/(2*NbX))*(2*M+1));
mat_cos2=cos(((pi*Q')/(2*NbY))*(2*N+1));

imout=[];
total=NbX*NbY;
inc=0;

alphap=sqrt(2/NbX)*ones(NbX,NbY);
alphap(1,:)=1/sqrt(NbX);
alphaq=sqrt(2/NbY)*ones(NbY,NbY);
alphaq(1,:)=1/sqrt(NbY);

imout=mat_cos1'*(alphap.*(A*(alphaq.*mat_cos2)));